/*
 *  SSL-Explorer
 *
 *  Copyright (C) 2003-2006 3SP LTD. All Rights Reserved
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2 of
 *  the License, or (at your option) any later version.
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public
 *  License along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
			
package com.sslexplorer.activedirectory;

import java.util.Collection;

public final class UserContainer extends PrincipalContainer<ActiveDirectoryUser> {
    private static final String USERS_CACHE_PREFIX = "users";
    private static final String CACHE_FULL_MESSAGE = "activeDirectory.cache.user.full";
    public static final UserContainer EMPTY_CACHE = new UserContainer(0, true, false);
    private final boolean usernamesAreCaseSensitive;
    
    UserContainer(int cacheSize, boolean inMemoryCache, boolean usernamesAreCaseSensitive) {
        super(cacheSize, inMemoryCache, USERS_CACHE_PREFIX, CACHE_FULL_MESSAGE);
        this.usernamesAreCaseSensitive = usernamesAreCaseSensitive;
    }

    @Override
    synchronized boolean containsPrincipal(String principalName) {
        principalName = fixUpUsername(principalName);
        return super.containsPrincipal(principalName);
    }

    @Override
    synchronized ActiveDirectoryUser retrievePrincipal(String principalName) {
        principalName = fixUpUsername(principalName);
        return super.retrievePrincipal(principalName);
    }

    @Override
    synchronized Collection<ActiveDirectoryUser> retrievePrincipals(String filter) {
        filter = fixUpUsername(filter);
        return super.retrievePrincipals(filter);
    }

    @Override
    synchronized String storePrincipal(ActiveDirectoryUser principal) {
        String fixUpUsername = fixUpUsername(principal.getPrincipalName());
        storePrinciple(fixUpUsername, principal);
        return fixUpUsername;
    }

    @Override
    void removePrincipal(String principleName) {
        String fixUpUsername = fixUpUsername(principleName);
        super.removePrincipal(fixUpUsername);
    }

    private String fixUpUsername(String username) {
        return usernamesAreCaseSensitive ? username : username.toLowerCase();
    }
}